@@ -1,4 +1,4 @@
-/* 
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -20,11 +20,24 @@ package org.apache.felix.framework.util;
 
 import java.security.AccessController;
 import java.security.PrivilegedAction;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.EventListener;
+import java.util.EventObject;
 
-import org.apache.felix.framework.Felix;
 import org.apache.felix.framework.Logger;
-import org.osgi.framework.*;
+import org.osgi.framework.AllServiceListener;
+import org.osgi.framework.Bundle;
+import org.osgi.framework.BundleEvent;
+import org.osgi.framework.BundleListener;
+import org.osgi.framework.Constants;
+import org.osgi.framework.Filter;
+import org.osgi.framework.FrameworkEvent;
+import org.osgi.framework.FrameworkListener;
+import org.osgi.framework.ServiceEvent;
+import org.osgi.framework.ServiceListener;
+import org.osgi.framework.ServicePermission;
+import org.osgi.framework.ServiceReference;
+import org.osgi.framework.SynchronousBundleListener;
 
 public class EventDispatcher
 {
@@ -48,10 +61,10 @@ public class EventDispatcher
 
     // A single thread is used to deliver events for all dispatchers.
     private static Thread m_thread = null;
-    private static String m_threadLock = "thread lock";
+    private static String m_threadLock = new String("thread lock");
     private static int m_references = 0;
-    private static boolean m_stopping = false;
-    private static boolean m_stopped = false;
+    private static volatile boolean m_stopping = false;
+
     // List of requests.
     private static final ArrayList m_requestList = new ArrayList();
     // Pooled requests to avoid memory allocation.
@@ -69,12 +82,29 @@ public class EventDispatcher
         synchronized (m_threadLock)
         {
             // Start event dispatching thread if necessary.
-            if (m_thread == null)
+            if (m_thread == null || !m_thread.isAlive())
             {
+                m_stopping = false;
+
                 m_thread = new Thread(new Runnable() {
                     public void run()
                     {
-                        EventDispatcher.run();
+                        try
+                        {
+                            EventDispatcher.run();
+                        }
+                        finally 
+                        {
+                             // Ensure we update state even if stopped by external cause
+                            // e.g. an Applet VM forceably killing threads
+                            synchronized (m_threadLock)
+                            {
+                                m_thread = null;
+                                m_stopping = false;
+                                m_references = 0;
+                                m_threadLock.notifyAll();
+                            }
+                        }
                     }
                 }, "FelixDispatchQueue");
                 m_thread.start();
@@ -82,19 +112,17 @@ public class EventDispatcher
 
             // reference counting and flags
             m_references++;
-            m_stopping = false;
-            m_stopped = false;
         }
 
         return eventDispatcher;
     }
-    
+
     public static void shutdown()
     {
         synchronized (m_threadLock)
         {
-            // Return if already stopped.
-            if (m_stopped)
+            // Return if already dead or stopping.
+            if (m_thread == null || m_stopping)
             {
                 return;
             }
@@ -105,16 +133,20 @@ public class EventDispatcher
             {
                 return;
             }
-            
-            // Signal dispatch thread.
-            m_stopping = true;
-            synchronized (m_requestList)
-            {
-                m_requestList.notify();
-            }
 
-            // Wait for dispatch thread to stop.
-            while (!m_stopped)
+            m_stopping = true;
+        }
+
+        // Signal dispatch thread.
+        synchronized (m_requestList)
+        {
+            m_requestList.notify();
+        }
+
+        // Use separate lock for shutdown to prevent any chance of nested lock deadlock
+        synchronized (m_threadLock)
+        {
+            while (m_thread != null)
             {
                 try
                 {
@@ -124,9 +156,6 @@ public class EventDispatcher
                 {
                 }
             }
-            
-            // remove the thread reference
-            m_thread = null;
         }
     }
 
@@ -445,7 +474,7 @@ public class EventDispatcher
             {
                 listeners = m_serviceListeners;
             }
-    
+
             // See if the listener is already registered, if so then
             // handle it according to the spec.
             for (int i = 0; i < listeners.length; i += LISTENER_ARRAY_INCREMENT)
@@ -528,8 +557,9 @@ public class EventDispatcher
     private void fireEventAsynchronously(
         Logger logger, int type, Object[] listeners, EventObject event)
     {
+        //TODO: should possibly check this within thread lock, seems to be ok though without
         // If dispatch thread is stopped, then ignore dispatch request.
-        if (m_stopped || m_stopping)
+        if (m_stopping || m_thread == null)
         {
             return;
         }
@@ -685,7 +715,7 @@ public class EventDispatcher
         if (objectClass != null)
         {
             boolean hasPermission = false;
-            
+
             Object sm = System.getSecurityManager();
             if ((acc != null) && (sm != null))
             {
@@ -693,15 +723,15 @@ public class EventDispatcher
                     !hasPermission && (i < objectClass.length);
                     i++)
                 {
-                    try 
+                    try
                     {
                         ServicePermission perm =
                             new ServicePermission(
                                 objectClass[i], ServicePermission.GET);
                         ((SecurityManager) sm).checkPermission(perm, acc);
                         hasPermission = true;
-                    } 
-                    catch (Exception ex) 
+                    }
+                    catch (Exception ex)
                     {
                     }
                 }
@@ -773,11 +803,6 @@ public class EventDispatcher
                 // has been called then exit, otherwise dispatch event.
                 if ((m_requestList.size() == 0) && (m_stopping))
                 {
-                    synchronized (m_threadLock)
-                    {
-                        m_stopped = true;
-                        m_threadLock.notifyAll();
-                    }
                     return;
                 }
 
